# Mira Database Server

[![NPM version][npm-image]][npm-url]
[![NPM downloads][downloads-image]][downloads-url]

MiraDB super fast database server.


## All Example Source Codes (client & Server)
https://github.com/Nodeclient/Mira-DB   

## Installation

```
npm install mira-db --save
```

## Command Examples
    SELECT TABLE <TABLE_NAME>; 
    SELECT TABLE <TABLE_NAME> COUNT; 
    SELECT TABLE <TABLE_NAME> COLUMN ["COL1"];
    SELECT TABLE <TABLE_NAME> COLUMN ["COL1"] FIND ["STRING"]; 
    ADD ROW <TABLE_NAME> COLUMN ["COL1","COL2","COL3"] VALUE ["VAL1","VAL2","VAL3"]; 
    ADD COLUMN COL1,COL2,COL3 TABLE <TABLE_NAME>; 
    RENAME TABLE ["TABLE"] VALUE ["NEW_NAME"]; 
    RENAME DATABASE ["DATABASE_NAME"] VALUE ["NEW_NAME"]; 
    RENAME COLUMN ["COL1","COL2"] TABLE <TABLE_NAME> VALUE ["NEW_1","NEW_2"]; 
    DELETE COLUMN <COLUMN_NAME> TABLE <TABLE_NAME>; 
    DELETE ROW <TABLE_NAME> COLUMN <COLUMN_NAME> VALUE ["STRING"];
    CREATE TABLE <TABLE_NAME> COLUMN ["COL1","COL2","COL3"] VALUE ["VAL1","VAL2","VAL3"];
    CREATE DATABASE <DATABASE_NAME>;
    DROP DATABASE <DATABASE_NAME>;
    DROP TABLE <TABLE_NAME>;
    LIST DATABASE;
    LIST TABLE <DATABASE_NAME>;

## How to use
    1)  set a new folder (fullpath)
        *  var My_Db_folder = "your_database_folder";
    2)  Create a Database
        * CREATE DATABASE test;
    3)  Create new table
        * CREATE TABLE person COLUMN ["name","surname","mail"] VALUE ["leo","zacharry","leo.zacharry@gmail.com"];
    4)  Select your table
        * SELECT TABLE person; 
    5)  get result
        var result = mydb.Query(<QUERY>,<DB>,<PERMISSION>);
        console.log(result);       

## Nodejs  >  local_server.js
 ```js
    // INCLUDE MIRA
    const jsql = require("mira-db");
    // SET YOUR USER PERMISSION
    var perms = { SELECT:true,ADD:true,RENAME:true,DROP:true,DELETE:true,CREATE:true,LIST:true }
    // YOUR STORAGE FOLDER FULL PATH
    var My_Db_folder = "/your_database_folder";
    //SELECT DATABASE
    var My_Db_name = "test";
    // CREATE NEW DATABASE OBJECT
    var mydb = new jsql(My_Db_folder,"UTF-8");
    // SET QUERY
    var query = 'SELECT TABLE person';
    //DATA RESULT
    var result = mydb.Query(query,My_Db_name,perms);
    console.log(  result );
```

## Nodejs  >  remote_server.js
    ```
    npm install socket.io
    ```
 ```js
const io = require("socket.io"), 
      user = require('mira-db/userdata'),
      DB = require('mira-db'),
server = io.listen(8123);

var UserData = new Map();     
var UserPerm = new Map();  

var Storage_Folder =  process.cwd() + "/data";
var database = new DB(Storage_Folder,"UTF-8");

server.on("connection", (socket) => {
    var connect_time = new Date();
    UserData.set(socket.id, true); 

    socket.on("disconnect", () => { 
        UserData.delete(socket.id); 
        UserPerm.delete(socket.id);
    });

    socket.emit("EventData",[connect_time,"CONNECTION_SUCCESS"]);
    
    socket.on("query", (data) => {
        queryKey = UserData.get(data["id"]);
        PermKey = UserPerm.get(data["id"])[0];
        DBKey = UserPerm.get(data["id"])[1];
        var PermsData = user.LoadPermsData(PermKey+".perm");
    
        if(queryKey){
            var result;
            if(DBKey=="*"){
                result = database.Query(data["query"],data["db"],PermsData);
            } else {
                result = database.Query(data["query"],DBKey,PermsData);
            }
            socket.emit("QueryData",result);
        } else {
            socket.emit("QueryData",[data,"LOGIN_ERROR","CANNOT ACCESS DATABASE"]);
        }
    });


    socket.on("Login", (u) => {
        var LoginData = user.LoadLoginData("user.conf"); 
        var result = user.Login(u["user"], u["pass"],LoginData) ;
            if (result){
                UserPerm.set(socket.id,[result["perm"],result["db"]]);
                socket.emit("EventData",[true,"LOGIN_SUCCESS","",socket.id]);
            }else{
                socket.emit("EventData",[false,"LOGIN_ERROR","INVALID USERNAME OR PASSWORD",0]);
            }
    });

});
```

####Updates & Fixed
+ Version 1.0.4
    + ADD example remote server 
    + ADD example local database
    + ADD example client source
+ Version 1.0.3
    + fixed generic code
+ Version 1.0.2
    + generic change and fixed "FIND" command
    + fixed (handle errors)
    + fixed "LIST" command

## License

MIT

[npm-image]: https://img.shields.io/npm/v/mira-db.svg?style=flat
[npm-url]: https://npmjs.org/package/mira-db
[downloads-image]: https://img.shields.io/npm/dm/mira-db.svg?style=flat
[downloads-url]: https://npmjs.org/package/mira-db